import sys
sys.path.insert(1, "../../../lib")
sys.path.insert(1, "../../../Python")


import rumd
# it is essential that there is no doubt about which version is actually run!
print "File used to import the rumd module", rumd.__file__

from rumd.Simulation import Simulation
from rumd.RUMD_Test import RUMD_Test

# Run-time data analysis function
class ParticleZero:
    def __init__(self):
        pass

    def PZ(self, sample):
        """Callback function, which must return a string"""
        # get particle zero 's position
        pos = sample.GetPositions()[0]
        vel = sample.GetVelocities()[0]
        
        return "%.5f %.5f %.5f %.5f %.5f %.5f" % (pos[0], pos[1], pos[2],
                                                    vel[0], vel[1], vel[2])


class Test_IPL_12 (RUMD_Test):
    """
    Single component system with IPL_12 potential and NVE dynamics.
    """
    def __init__(self):
        RUMD_Test.__init__(self)
        self.numParticlesList = [512, 1000, 8000]
    
    def RunSimulation(self, mode, numParticles, pb, tp):

        # set the number of time-steps depending on the mode (assigned
        # as a class attribute so the calling function can access it later)
        self.numTimeSteps = {"short":          100,
                             "performance": 100000,
                             "stress":   100000000}[mode]
        
        # sample and simulation
        startConfg = "startN%d.xyz.gz" % numParticles

        
        sim = Simulation(startConfg, pb, tp, verbose=False)

        if self.generate_reference_data:
            sim.SetOutputDirectory(self.ref_directory)
            sim.SetSortInterval(0)
        else:
            sim.SetOutputDirectory(self.out_directory)

        # integrator
        itg = rumd.IntegratorNVT(timeStep=0.005, targetTemperature=0.8)

        # Create IPL pair-potential
        potential = rumd.Pot_IPL_12(cutoff_method=rumd.ShiftedPotential)
        # i  j   Sig   Eps  Rcut (in units of Sigma_ij)
        potential.SetParams(0, 0, 1.00, 1.00, 2.5)
        
    
        sim.SetIntegrator(itg)
        sim.SetPotential(potential)


        # Register a callback function with the simulation object
        # first create an object of type ParticleZero
        analysis_obj = ParticleZero()
        
        # create an output manager, the string is a label and the base for the filenames
        sim.NewOutputManager("particleZero")
        

        sim.SetOutputScheduling("particleZero", "linear", interval=1)
        
        # add the callback function to the output manager
        sim.RegisterCallback("particleZero", analysis_obj.PZ, header="x y z vx vy vz" )
        


        # tune output depending on the mode
        if mode == "short":
            sim.SetOutputScheduling("energies","linear",interval=1)
            sim.SetOutputScheduling("trajectory","linear",interval=10)
            sim.SetOutputScheduling("particleZero","linear",interval=1)
            sim.SetBlockSize(50)
        elif mode == "stress":
            sim.SetOutputScheduling("energies","linear",interval=5000)
            sim.SetOutputScheduling("particleZero","linear",interval=1000)
        else:
            assert mode == "performance"
            print "Note there is no output directory for performance test--the only output is the timing file"
            
        sim.Run(self.numTimeSteps,
                suppressAllOutput=(mode == "performance"))



    def CompareOutputDirect(self):
        """ Return non-zero if differences occur in output files"""
        num_diffs = self.CompareTrajectoryFiles()
        num_diffs += self.CompareEnergiesFiles()
        num_diffs += self.CompareEnergiesFiles("particleZero")
        return num_diffs

if __name__ == "__main__":

    rumd_test = Test_IPL_12()
    
    # standard short test to be called from makefile
    # For generating reference data and running stress and performance modes
    # there are corresponding scripts to be submitted to pbs


    sys.exit(rumd_test.RunShort("gen_ref" in sys.argv ) )
    
    # for individual runs, could do for example
    #sys.exit(rumd_test.Test("short", 1000, 16, 8))
